<?php

namespace SupabloxPro\Block_Supports;

\WP_Block_Supports::get_instance()->register(
	'supabloxProSpacing',
	[
		'register_attribute' => __NAMESPACE__ . '\register_spacing_support',
		'apply' => __NAMESPACE__ . '\apply_spacing_support',
	]
);

add_filter( 'render_block_data', __NAMESPACE__ . '\spacing_render_block_data', 10 );

function register_spacing_support( \WP_Block_Type $block_type ) {
	if ( ! block_has_support( $block_type, 'supabloxProSpacing', false ) ) {
		return;
	}

	if ( ! array_key_exists( 'style', $block_type->attributes ) ) {
		$block_type->attributes['style'] = [ 'type' => 'object' ];
	}

	$all_attributes = [
		'marginTop' => [
			'type' => 'string',
			'output' => false,
			'property' => 'margin-top',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'marginRight' => [
			'type' => 'string',
			'output' => false,
			'property' => 'margin-right',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'marginBottom' => [
			'type' => 'string',
			'output' => false,
			'property' => 'margin-bottom',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'marginLeft' => [
			'type' => 'string',
			'output' => false,
			'property' => 'margin-left',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'paddingTop' => [
			'type' => 'string',
			'output' => false,
			'property' => 'padding-top',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'paddingRight' => [
			'type' => 'string',
			'output' => false,
			'property' => 'padding-right',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'paddingBottom' => [
			'type' => 'string',
			'output' => false,
			'property' => 'padding-bottom',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'paddingLeft' => [
			'type' => 'string',
			'output' => false,
			'property' => 'padding-left',
			'should_output_callback' => __NAMESPACE__ . '\should_output_spacing_property',
		],
		'rowGap' => [
			'type' => 'string',
			'output' => false,
			'property' => 'row-gap',
		],
		'columnGap' => [
			'type' => 'string',
			'output' => false,
			'property' => 'column-gap',
		],
	];
	$all_sides = [ 'top', 'right', 'bottom', 'left' ];
	$options = $block_type->supports[ 'supabloxProSpacing' ];
	$attributes = [];

	if ( is_array( $options ) ) {
		foreach ( [ 'margin', 'padding' ] as $key ) {
			if ( isset( $options[ $key ] ) && $options[ $key ] !== false ) {
				$sides = $options[ $key ] === true ? $all_sides : $options[ $key ];
				foreach ( $sides as $side ) {
					$attr_name = $key . ucfirst( $side );
					if ( isset( $all_attributes[ $attr_name ] ) ) {
						$attributes[ $attr_name ] = $all_attributes[ $attr_name ];
					}
				}
			}
		}
		if ( isset( $options['blockGap'] ) && $options['blockGap'] !== false ) {
			$sides = $options['blockGap'] === true ? [ 'vertical' ] : $options['blockGap'];
			if ( in_array( 'vertical', $sides, true ) ) {
				$attributes['columnGap'] = $all_attributes['columnGap'];
			}
			if ( in_array( 'horizontal', $sides, true ) ) {
				$attributes['rowGap'] = $all_attributes['rowGap'];
			}
		}
	}

	style_engine()->register( $block_type, $attributes );
}

function apply_spacing_support( \WP_Block_Type $block_type, $block_attributes ) {
	if ( ! block_has_support( $block_type, 'supabloxProSpacing', false ) ) {
		return;
	}

	$attributes = [];
	$block_styles = $block_attributes['style'] ?? null;

	if ( empty( $block_styles ) ) {
		return $attributes;
	}

	$has_padding_support = block_has_support( $block_type, [ 'supabloxProSpacing', 'padding' ], false );
	$has_margin_support	 = block_has_support( $block_type, [ 'supabloxProSpacing', 'margin' ], false );
	$skip_padding = should_skip_block_supports_serialization( $block_type, 'supabloxProSpacing', 'padding' );
	$skip_margin = should_skip_block_supports_serialization( $block_type, 'supabloxProSpacing', 'margin' );
	$spacing_block_styles = [ 'margin' => null, 'padding' => null ];

	if ( $has_padding_support && ! $skip_padding ) {
		$spacing_block_styles['padding'] = $block_styles['spacing']['padding'] ?? null;
	}

	if ( $has_margin_support && ! $skip_margin ) {
		$spacing_block_styles['margin'] = $block_styles['spacing']['margin'] ?? null;
	}

	if ( empty( $spacing_block_styles ) ) {
		return $attributes;
	}

	$styles = wp_style_engine_get_styles( [ 'spacing' => $spacing_block_styles ] );

	if ( ! empty( $styles['css'] ) ) {
		$attributes['style'] = $styles['css'];
	}

	return $attributes;
}

/**
 * Don't output spacing properties for default breakpoint and state.
 * WordPress already add them as inline styles.
 */
function should_output_spacing_property( $value, string $breakpoint, string $state ) {
	return ! ( $breakpoint === 'lg' && $state === 'default' );
}

function spacing_render_block_data( array $parsed_block ) {
	if ( empty( $parsed_block['blockName'] ) ) return $parsed_block;
	if ( isset( $parsed_block['attrs']['supablox'] ) ) return $parsed_block;

	$block_type = \WP_Block_Type_Registry::get_instance()->get_registered( $parsed_block['blockName'] );

	if ( ! block_has_support( $block_type, 'supabloxProSpacing', false ) ) {
		return $parsed_block;
	}

	$style_attributes = style_engine()->registry->get_registered( $parsed_block['blockName'] );

	foreach ( [ 'margin', 'padding' ] as $style_key ) {
		if ( isset( $parsed_block['attrs']['style']['spacing'][ $style_key ] ) ) {
			foreach ( $parsed_block['attrs']['style']['spacing'][ $style_key ] as $side => $value ) {
				$spacing_key = $style_key . ucfirst( $side );
				$spacing_attribute = $style_attributes[ $spacing_key ] ?? null;

				if ( ! $spacing_attribute ) continue;

				if ( is_null( $spacing_attribute->get_value( $parsed_block['attrs'], 'lg', 'default' ) ) ) {
					$spacing_attribute->set_value( $parsed_block['attrs'], 'lg', 'default', $value );
				}
			}
		}
	}

	return $parsed_block;
}
