<?php

namespace SupabloxPro\Packages\Admin;

class Features {

	private static $instance = null;

	private array $registered = [];

	public function get_registered() {
		return $this->registered;
	}

	public function is_enabled( string $id ) {
		return ! empty( get_options()['features'][ $id ] );
	}

	public function register( string $id, array $feature ) {
		$this->registered[ $id ] = wp_parse_args(
			$feature,
			[
				'status' => '',
				'title' => '',
				'description' => '',
				'requires_php' => null,
				'requires_at_least' => null,
				'links' => [],
			]
		);
	}

	public function add_settings() {
		if ( ! empty( $this->registered ) ) {
			add_settings_field( 'features', 'Features', [ $this, 'features_setting_callback' ], Settings_Page::ID, PACKAGE_NAME );
		}
	}

	public function features_setting_callback() {
		$groups = [
			'stable' => [
				'settings' => [],
			],
			'unstable' => [
				'settings' => [],
			],
		];

		foreach ( $this->registered as $id => $feature ) {
			if ( $feature['status'] ) {
				$groups['unstable']['settings'][ $id ] = $feature;
			} else {
				$groups['stable']['settings'][ $id ] = $feature;
			}
		}

		foreach ( $groups as $group_id => $group ) {
			if ( empty( $group['settings'] ) ) continue;

			if ( ! empty( $groups['stable']['settings'] ) && $group_id !== 'stable' ) {
				echo '<hr style="margin-block:1em;"></hr>';
			}

			if ( ! empty( $group['label'] ) ) {
				echo '<h4 style="margin-block:0.4em 1em;">' . esc_html( $group['label'] ) . '</h4>';
				if ( ! empty( $group['description'] ) ) {
					echo '<p class="description" style="margin-block:-1em 1em;">' . esc_html( $group['description'] ) . '</p>';
				}
			}

			echo '<fieldset>';
			echo '<legend class="screen-reader-text">Features</legend>';
			echo '<div style="display:flex;flex-direction:column;gap:8px;">';
			foreach ( $group['settings'] as $id => $feature ) {
				$this->render_checkbox( $id, $feature );
			}
			echo '</div>';
			echo '</fieldset>';
		}
	}

	public function settings_sanitize_callback( $input ) {
		return [
			'features' => array_map( 'boolval', $input['features'] ?? [] ),
		];
	}

	private function render_checkbox( string $id, array $feature ) {
		global $wp_version;

		$value = $this->is_enabled( $id ) ? 1 : 0;

		$is_php_supported = isset( $feature['requires_php'] ) ? version_compare( PHP_VERSION, $feature['requires_php'], '>=' ) : true;
		$is_wp_supported  = isset( $feature['requires_at_least'] ) ? version_compare( $wp_version, $feature['requires_at_least'], '>=' ) : true;
		$disabled = ! $is_php_supported || ! $is_wp_supported;

		echo '<div style="display:flex;align-items:top;gap:4px;">';
		echo '<input type="checkbox" name="' . esc_attr( PACKAGE_NAME ) . '[features][' . esc_attr( $id ) . ']" id="' . esc_attr( $id ) . '" value="1"' . ( $value ? ' checked' : '' ) . ( $disabled ? ' disabled' : '' ) . ' style="margin-top:6px;">';
		echo '<div>';
		echo '<label for="' . esc_attr( $id ) . '">';
		echo ' <strong>' . esc_html( $feature['title'] ) . '</strong>';
		if ( $feature['status'] ) {
			echo $this->badge( $feature['status'] ); // phpcs:ignore WordPress.Security.EscapeOutput
		}
		echo '</label>';

		if ( ! empty( $feature['description'] ) ) {
			echo '<p class="description" style="margin:-0.5em 1.0em 0.25em 0;">' . esc_html( $feature['description'] ) . '</p>';
		}

		if ( $value && ! empty( $feature['links'] ) ) {
			for ( $i = 0; $i < count( $feature['links'] ); $i++ ) { // phpcs:ignore
				if ( $i > 0 ) echo ' | ';
				echo '<a';
				echo ' href="' . esc_url_raw( $feature['links'][ $i ]['url'] ) . '"';
				if ( ! empty( $feature['links'][ $i ]['target'] ) ) {
					echo ' target="' . esc_attr( $feature['links'][ $i ]['target'] ) . '"';
					if ( $feature['links'][ $i ]['target'] === '_blank' ) {
						echo ' rel="noopener noreferrer"';
					}
				}
				echo '>';
				echo esc_html( $feature['links'][ $i ]['title'] );
				echo '</a>';
			}
		}

		if ( $disabled ) {
			if ( ! $is_php_supported ) {
				// translators: %s: PHP version.
				echo '<p class="description"><i class="dashicons dashicons-info"></i> ' . sprintf( esc_html__( 'This feature requires PHP version %s or higher.', 'supablox' ), esc_attr( $feature['requires_php'] ) ) . '</p>';
			}
			if ( ! $is_wp_supported ) {
				// translators: %s: WordPress version.
				echo '<p class="description"><i class="dashicons dashicons-info"></i> ' . sprintf( esc_html__( 'This feature requires WordPress version %s or higher.', 'supablox' ), esc_attr( $feature['requires_at_least'] ) ) . '</p>';
			}
		}

		echo '</div>';
		echo '</div>';
	}

	private function badge( string $text ) {
		return '<span style="margin-left:6px;padding:3px 6px;font-size:75%;font-weight:normal;border-radius:3px;color:#777;background:rgba(0,0,0,0.07);">' . esc_html( $text ) . '</span>';
	}

	public static function get_instance(): static {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}
}
