<?php

namespace SupabloxPro\Packages\Style_Engine;

class Inline_Styles {

	private string $style_handle;
	private string $block_attribute_name;
	private string $css_property_prefix;

	public function __construct(
		private array $options,
		private Registry $registry,
		private Group_Hover $group_hover,
		private CSS_Utils $css_utils
	) {
		$this->style_handle = $options['id'] . '-block-supports';
		$this->block_attribute_name = $options['block_attribute_name'];
		$this->css_property_prefix = $options['css_property_prefix'];

		add_action( 'init', [ $this, 'register_style' ] );
		add_filter( 'render_block', [ $this, 'render_block' ], 9, 3 );
		add_action( 'wp_enqueue_scripts', [ $this, 'process_enqueued_styles' ] );
		add_action( 'wp_footer', [ $this, 'process_enqueued_styles' ] );
	}

	public function register_style() {
		$dependencies = $this->options['style_dependencies'] ?: [];

		wp_register_style( $this->style_handle, false, [ 'core-block-supports', ...$dependencies ] );
	}

	public function render_block( string $block_content, array $block, \WP_Block $block_instance ) {
		if (
			! isset( $block['blockName'] ) ||
			empty( $block_instance->block_type->attributes[ $this->block_attribute_name ] )
		) {
			return $block_content;
		}

		$style_registry = $this->registry;
		$css_utils = $this->css_utils;
		$group_hover = $this->group_hover;
		$style_attributes = $style_registry->get_registered( $block['blockName'] );

		if ( empty( $style_attributes ) || ! is_array( $block['attrs'] ) ) {
			return $block_content;
		}

		$style_registry->enqueue_style( $block['blockName'], $style_attributes );

		$html_tags = new \WP_HTML_Tag_Processor( $block_content );

		if ( $html_tags->next_tag() ) {
			$custom_class_name = '';

			$html_tags->add_class( $css_utils->class_name( 'block' ) );

			if ( isset( $block['attrs'][ $group_hover->attribute_name ] ) ) {
				$html_tags->add_class( $block['attrs'][ $group_hover->attribute_name ] );
			}

			if ( isset( $block['attrs'][ $this->block_attribute_name ]['visibility'] ) ) {
				if ( ! $custom_class_name ) {
					$custom_class_name = $this->inline_styles_get_customized_class_name();
				}
			}

			foreach ( $style_attributes as $style_attribute ) {
				if ( $style_attribute->is_customized( $block['attrs'] ) ) {
					if ( $style_attribute->class_name ) {
						$html_tags->add_class( $style_attribute->class_name );
					}
					if ( ! $custom_class_name ) {
						$custom_class_name = $this->inline_styles_get_customized_class_name();
					}
				}
			}

			if ( $custom_class_name ) {
				$html_tags->add_class( $custom_class_name );
				$style_registry->enqueue_customized_style( $custom_class_name, $block_instance, $style_attributes );
			}
		}

		return $html_tags->get_updated_html();
	}

	public function process_enqueued_styles() {
		if ( empty( $this->registry->enqueued_styles ) ) {
			return;
		}

		$widths = Devices::get_widths();
		$ids = Devices::get_ids();
		$default_index = Devices::get_default_index();
		$index = $default_index;
		$count = count( $widths );
		$result = [];

		// CSS generation strategy:
		// 1. Default breakpoint ('lg') is rendered first without a media query.
		// 2. Smaller breakpoints are generated in descending order using 'max-width' media queries.
		// 3. Larger breakpoints are generated in ascending order using 'min-width' media queries.
		while ( $index < $count ) {
			$id = $ids[ $index ];
			$width = $widths[ $id ];
			$rulesets = array_merge_recursive(
				$this->get_default_rulesets( $id ),
				$this->get_customized_rulesets( $id )
			);
			$visibility_rulesets = $this->inline_styles_get_visibility_rulesets( $id );

			if ( $index === $default_index ) {
				$min_width = $widths[ $ids[ $index - 1 ] ];
				$result[] = $this->css_utils->rulesets( $rulesets );
				$result[] = $this->css_utils->at_rule( 'media', [ 'min-width' => $min_width . 'px' ], $visibility_rulesets );
			} elseif ( $width && $index < $default_index ) {
				if ( $index > 0 ) {
					$min_width = $widths[ $ids[ $index - 1 ] ];
					$result[] = $this->css_utils->at_rule( 'media', [ 'max-width' => ( $width - 1 ) . 'px' ], $rulesets );
					// phpcs:ignore WordPress.Arrays.ArrayDeclarationSpacing.AssociativeArrayFound
					$result[] = $this->css_utils->at_rule( 'media', [ 'min-width' => $min_width . 'px', 'max-width' => ( $width - 1 ) . 'px' ], $visibility_rulesets );
				} else {
					$result[] = $this->css_utils->at_rule( 'media', [ 'max-width' => ( $width - 1 ) . 'px' ], array_merge_recursive( $rulesets, $visibility_rulesets ) );
				}
			} elseif ( $width ) {
				$result[] = $this->css_utils->at_rule( 'media', [ 'min-width' => $width . 'px' ], array_merge_recursive( $rulesets, $visibility_rulesets ) );
			}

			if ( $index === 0 ) $index = $default_index + 1;
			elseif ( $index > $default_index ) $index++;
			else $index--;
		}

		if ( $result ) {
			if ( ! wp_style_is( $this->style_handle ) ) {
				wp_enqueue_style( $this->style_handle );
			}
			wp_add_inline_style( $this->style_handle, implode( CSS_EOL, array_map( 'trim', $result ) ) );
		}

		$this->registry->enqueued_styles = [];
		$this->registry->enqueued_customized_styles = [];
	}

	public function get_default_block_type_css() {
		$data = array_fill_keys( Devices::get_ids(), '' );

		$this->registry->enqueued_styles = $this->registry->get_all_registered();

		foreach ( Devices::get_ids() as $breakpoint ) {
			$rulesets = $this->get_default_rulesets( $breakpoint );
			$data[ $breakpoint ] .= $this->css_utils->rulesets( $rulesets );
		}

		$this->registry->enqueued_styles = [];

		return $data;
	}

	public function get_default_rulesets( $breakpoint = 'lg' ) {
		$rulesets = [];

		foreach ( $this->registry->enqueued_styles as $block_name => $style_attributes ) {
			$class_name = wp_get_block_default_classname( $block_name );

			foreach ( $style_attributes as $style_attribute ) {
				if ( ! $style_attribute->output ) {
					continue;
				}

				$property = $style_attribute->property;
				$value = $style_attribute->get_default_value( $breakpoint, 'default', 'output' );

				$rulesets[ ".$class_name" ][ $property ] = $value;
			}
		}

		return $rulesets;
	}

	private function get_customized_rulesets( $media = 'lg' ) {
		$rulesets = [];

		foreach ( Registry::$states as $state ) {
			foreach ( $this->registry->enqueued_customized_styles as $class_name => $enqueued ) {
				foreach ( $enqueued[1] as $style_attribute ) {
					$property = $style_attribute->property;
					$value = $style_attribute->get_value(
						$enqueued[0]->parsed_block['attrs'],
						$media,
						$state,
						'output'
					);

					if ( is_callable( $style_attribute->should_output_callback ) ) {
						if ( ! call_user_func( $style_attribute->should_output_callback, $value, $media, $state ) ) {
							continue;
						}
					}

					if ( ! empty( $value ) && ! str_starts_with( $property, $this->css_property_prefix ) ) {
						$value .= ' !important';
					}

					if ( $state === 'hover' ) {
						if ( str_starts_with( $property, $this->css_utils->property_name( 'transition' ) ) ) {
							// Hover transition are applied to the default state as well
							// to use the same transition when leaving the hover state.
							$rulesets[ ".$class_name" ][ $property ] = $value;
						}
						$rulesets[ ".$class_name:hover" ][ $property ] = $value;
					} elseif ( $state === 'groupHover' ) {
						if ( $group_hover_class_name = $enqueued[0]->context[ $this->group_hover->context_name ] ?? null ) {
							if ( str_starts_with( $property,  $this->css_utils->property_name( 'transition' ) ) ) {
								$rulesets[ ":where(.$group_hover_class_name) .$class_name" ][ $property ] = $value;
							} else {
								$rulesets[ ":where(.$group_hover_class_name:hover) .$class_name" ][ $property ] = $value;
							}
						}
					} else {
						$rulesets[ ".$class_name" ][ $property ] = $value;
					}
				}
			}
		}

		return $rulesets;
	}

	private function inline_styles_get_visibility_rulesets( $media = 'lg' ) {
		$rulesets = [];

		foreach ( $this->registry->enqueued_customized_styles as $class_name => $enqueued ) {
			$visibility = $enqueued[0]->parsed_block['attrs'][ $this->block_attribute_name ]['visibility'] ?? null;

			if ( ! $visibility ) continue;

			$resolved_value = $this->registry->resolve_value( $visibility, $media, true );

			if ( ! $resolved_value ) {
				$rulesets[ ".$class_name" ]['display'] = 'none !important';
			}
		}

		return $rulesets;
	}

	private function inline_styles_get_customized_class_name() {
		return $this->css_utils->class_name( 'block-' . get_increment_for( 'block' ) );
	}
}
